import com.ibm.broker.javacompute.MbJavaComputeNode;
import com.ibm.broker.plugin.MbElement;
import com.ibm.broker.plugin.MbException;
import com.ibm.broker.plugin.MbGlobalMapSessionPolicy;
import com.ibm.broker.plugin.MbGlobalMap;
import com.ibm.broker.plugin.MbMessage;
import com.ibm.broker.plugin.MbMessageAssembly;
import com.ibm.broker.plugin.MbOutputTerminal;

public class SaveToCache extends MbJavaComputeNode {

	private String responseQueue;

	public void evaluate(MbMessageAssembly inAssembly) throws MbException {
		// Define the out terminal
		MbOutputTerminal out = getOutputTerminal("out");
		
		// Define the message assemblies
		MbMessage inMessage = inAssembly.getMessage();
		MbMessage outMessage = new MbMessage(inMessage);
		MbMessageAssembly outAssembly = new MbMessageAssembly(inAssembly,outMessage);

		// Get the original MsgId and ReplyToQ of the incoming request
		MbElement rootEl        = outAssembly.getMessage().getRootElement();
		MbElement replyToQEl    = rootEl.getFirstElementByPath("/MQMD/ReplyToQ");
		String    replyToQ      = replyToQEl.getValueAsString();
		
		MbElement msgIdEl       = rootEl.getFirstElementByPath("/MQMD/MsgId");
		String    msgId         = msgIdEl.getValueAsString();
		MbElement sourceQueueEl = rootEl.getFirstElementByPath("/MQMD/SourceQueue");
		String    sourceQueue   = sourceQueueEl.getValueAsString(); 
		/*
		 * We're about to overwrite the original ReplyToQ of this message.
		 * So, write it to the global cache it for safe keeping. This will
		 * be used subsequently by the Response message flow to send the
		 * response back to the original reply queue.
		 *
		 */
				
		/*
		 * Map 1  aliveUntilRestart
		 * write data to a map that will preserve the data until the IB node
		 * restarts ie no explicit data eviction policy - default is to keep 
		 * the data until restart of IB node.
		 * 
		 */ 
				
		MbGlobalMap globalMap=MbGlobalMap.getGlobalMap("aliveUntilRestart");
		globalMap.put(msgId, replyToQ);
				
		/* 
		 * Map 2 aliveFor60Seconds
		 * Write data to a map that will automatically expire after
		 * 60 seconds. The retrieveFromCache code will use this map to 
		 * restore the replyToQ
		 */
				
		MbGlobalMap LiveFor60=MbGlobalMap.getGlobalMap("aliveFor60Seconds", new MbGlobalMapSessionPolicy(60) );
		LiveFor60.put(msgId, replyToQ);
				
		/* 
		 * Map 3 aliveFor120Seconds
		 * Write data to a map that will automatically expire after 120
		 * seconds 
		 */		

		MbGlobalMap LiveFor120 = MbGlobalMap.getGlobalMap("aliveFor120Seconds", new MbGlobalMapSessionPolicy(120) );
		LiveFor120.put(msgId, replyToQ);

		System.out.println("Data to put in map [" + globalMap + "] using key [" + msgId + "]: " + replyToQ);

		// Figure out which instance we're in, by looking at original source queue
		String instance = "";
		for(int i = sourceQueue.length() - 1; i >= 0; i--) {
			String c = sourceQueue.substring(i, i+1);

			// not a digit? Done
			if(
				   !"0".equals(c)
				&& !"1".equals(c)
				&& !"2".equals(c)
				&& !"3".equals(c)
				&& !"4".equals(c)
				&& !"5".equals(c)
				&& !"6".equals(c)
				&& !"7".equals(c)
				&& !"8".equals(c)
				&& !"9".equals(c)
			) {
				break;
			}
			instance = c + instance;
		}
				
		// Now we set ReplyToQ to an appropriate value so that the subsequent retrieveCustomer flow will send the output to 
		// the right destination.
		// The ReplyToQ is actually going to be set based on the value of responseQueue, which
		// is a user-defined property of the Request Flow. 
		// There are two cases:
		// 1. The Response flow is located in the same broker as the Request (different EG)
		//     In this case, the retrieveCustomer flow puts the output on GLOBAL.CACHE.RESPONSE.IN, which is a local queue.
		// 2. The Response flow is located in a separate broker (BROKER2, using QMGR2). 
		//     In this case, the retrieveCustomer flow puts the output message on GLOBAL.CACHE.RESPONSE.REMOTE, which is
		//     a remote queue pointing to GLOBAL.CACHE.RESPONSE.IN in QMGR2. It can therefore be processed by a copy of the
		//     same message flow running in BROKER2.
		responseQueue = getUserDefinedAttribute("ResponseQueue").toString();
		outMessage.getRootElement().getFirstElementByPath("/MQMD/ReplyToQ").setValue(responseQueue + instance);
		
		// Go!
		out.propagate(outAssembly);
		
		return;
	}

}
