import com.ibm.broker.javacompute.MbJavaComputeNode;
import com.ibm.broker.plugin.MbException;
import com.ibm.broker.plugin.MbGlobalMap;
import com.ibm.broker.plugin.MbMessage;
import com.ibm.broker.plugin.MbMessageAssembly;
import com.ibm.broker.plugin.MbOutputTerminal;

public class XC10RetrieveFromCache extends MbJavaComputeNode {

	public void evaluate(MbMessageAssembly inAssembly) throws MbException {
		// Define out terminal
		MbOutputTerminal out = getOutputTerminal("out");
		
		// Define the message assemblies
		MbMessage inMessage = inAssembly.getMessage();
		MbMessage outMessage = new MbMessage(inMessage);
		MbMessageAssembly outAssembly = new MbMessageAssembly(inAssembly,outMessage);

		/*
		 * Get the current CorrelId (i.e. the msgId of the message to which THIS message
		 * is a reply). Ignore the MsgId!
		 */
		String msgId = inAssembly
			.getMessage()
			.getRootElement()
			.getFirstElementByPath("/MQMD/CorrelId")
			.getValueAsString()
		;

		// Now we can restore the original ReplyToQ by looking it up in the cache.
		/* TH?? old code (used for default global cache  
		 * MbGlobalMap globalMap = MbGlobalMap.getGlobalMap();
		 *
		 * String replyToQ = (String)globalMap.get(msgId);
		*/
		
		System.out.println("XC10Retrieve - about to get data ");
		// Now we can restore the original ReplyToQ by looking it up in the XC10 data grid.
        MbGlobalMap xc10Map = MbGlobalMap.getGlobalMap("userXXdatagrid","CS_XC10DATAGRID");
		
		String replyToQ = (String)xc10Map.get(msgId);
		System.out.println("XC10Retrieve - Data in xc10Map [" + xc10Map + "] key/ReplyQ [" + msgId + "]: " + replyToQ);

		/* don't remove data - leave clean up to Data eviction policy.
		xc10Map.remove(msgId); 
		*/
		
		
		// Set the ReplyToQ field
		outMessage.getRootElement().getFirstElementByPath("/MQMD/ReplyToQ").setValue(replyToQ);
		
		// Set the MsgId back to what it was before
		outMessage.getRootElement().getFirstElementByPath("/MQMD/MsgId").setValue(getBytes(msgId));
		
		// Set the QMGR to blank
		outMessage.getRootElement().getFirstElementByPath("/MQMD/ReplyToQMgr").setValue("        ");


		out.propagate(outAssembly);
		return;
	}

	/**
     * @param hex String containing an even number of hexadecimal digits e.g. "ac1f7800"
     * @return Returns the bytes described by those digits
     */
    private byte[] getBytes(String hex) {
    	int numBytes = hex.length() / 2;
        byte[] bytes = new byte[numBytes];
        for (int i = 0; i < numBytes; i ++) {
            bytes[i] = (byte) Integer.parseInt(hex.substring(2 * i, 2 * i + 2), 16);
        }
        return bytes;
    	
    }
}
